/*
 * battery.c
 *
 *  Created on: 10 Nov 2018
 *      Author: dadan
 */

#include "battery.h"

void battery_setup()
{
	// enable ADC
	ADCSRA = (1<<ADEN);

	// ADC reference - AVCC with external capacitor
	ADMUX |= (1<<REFS0);

	// ADC6
	ADMUX |= (1<<MUX1) | (1<<MUX2);

	// enable ADC interrupt request
	ADCSRA |= (1<<ADIE);

	// ADC prescaler 8 -> 1MHz/8 = 125 kHz
	ADCSRA |= (1<<ADPS0) | (1<<ADPS1);
}

void battery_start_measurement()
{
	// start ADC conversion
	ADCSRA |= (1<< ADSC);
}

uint16_t battery_get_voltage()
{
	uint32_t voltage = (g_battery_voltage * (uint32_t) 6600 ) / 1024;
	return voltage;
	/*
	if (ADCSRA & (1<<ADIF))
	{
		return ADC;
	}
	else return ADC;
	*/
}

uint8_t battery_get_percentage()
{
	// charge state in percent is calculated only from battery voltage


	uint16_t battery_adc = g_battery_voltage;

	if (battery_adc >= BATTERY_FULL_VALUE)
		return 100;

	if (battery_adc <= BATTERY_EMPTY_VALUE)
		return 0;

	return ((battery_adc - BATTERY_EMPTY_VALUE) * 100 ) / (BATTERY_FULL_VALUE - BATTERY_EMPTY_VALUE);
}

void battery_get_voltage_string(char str[], uint8_t len)
{
    uint16_t battery_voltage;

    if (len < 5)		// array is too short
    	return;

    battery_voltage = battery_get_voltage();

    str[0] = '0' + battery_voltage / 1000;
    battery_voltage -= (battery_voltage / 1000) * 1000;

    str[1] = '.';

    str[2] = '0' + battery_voltage / 100;
    battery_voltage -= (battery_voltage / 100) * 100;

    str[3] = '0' + battery_voltage / 10;
    battery_voltage -= (battery_voltage / 10) * 10;

    str[4] = 'V';
    str[5] = 0;
}

void battery_get_percentage_string(char str[], uint8_t len)
{
    uint8_t battery_percentage;

    if (len < 4)		// array is too short
    	return;


    battery_percentage = battery_get_percentage();

    str[0] = '0' + battery_percentage/ 100;
    battery_percentage -= (battery_percentage / 100) * 100;

    str[1] = '0' + battery_percentage / 10;
    battery_percentage -= (battery_percentage / 10) * 10;

    str[2] = '0' + battery_percentage % 10;

    str[3] = '%';

    str[4] = 0;
}
